//
//  TableViewController.swift
//  iCloudToDoExample
//
//  Created by Scott Lee on 2016. 10. 14..
//  Copyright © 2016년 Studio G.B.U. All rights reserved.
//

import UIKit
import CloudKit

class TableViewController: UITableViewController {
    
    struct todo {
        static var data: NSMutableArray = []
    }
    
    var totalTodo: Int? = nil
    let database = CKContainer.default().privateCloudDatabase
    
    class todoRecord {
        var title: String!
        var isComplete: Bool!
        
        init(title: String, isComplete: Bool) {
            self.title = title
            self.isComplete = isComplete
        }
    }
    
    // 새로 추가한 변수들
    var isSub = false
    var rowNum: Int!
    
    override func viewDidLoad() {
        let refreshControl = UIRefreshControl()
        refreshControl.tintColor = UIColor.blue
        refreshControl.addTarget(self, action: #selector(addTodo), for: UIControlEvents.valueChanged)
        self.refreshControl = refreshControl
        
        if isSub {
            navigationItem.title = ((todo.data[rowNum!] as! NSArray)[0] as! CKRecord).value(forKey: "title") as? String
        } else {
            iCloudAvailable()
            checkZoneAvailable()
        }
    }
    
    // Todo 자료 로드
    func todoDataLoad(recordType: String, record: CKRecord!) {
        switch recordType {
        case "todoRecord":
            let query = CKQuery(recordType: "todoRecord", predicate: NSPredicate(format: "TRUEPREDICATE"))
            query.sortDescriptors = [NSSortDescriptor(key: "creationDate", ascending: true)]
            let zoneID = CKRecordZoneID(zoneName: "todoZone", ownerName: CKCurrentUserDefaultName)
            self.refreshControl?.beginRefreshing()
            database.perform(query, inZoneWith: zoneID, completionHandler: loadHandler)
        case "todoSubRecord":
            let query = CKQuery(recordType: "todoSubRecord", predicate: NSPredicate(format: "ref = %@", record.recordID.recordName))
            query.sortDescriptors = [NSSortDescriptor(key: "creationDate", ascending: false)]
            let zoneID = CKRecordZoneID(zoneName: "todoSubZone", ownerName: CKCurrentUserDefaultName)
            database.perform(query, inZoneWith: zoneID, completionHandler: loadHandler)
        default:
            NSLog("해당하는 타입을 지원하지 않습니다.")
        }
    }
    
    // 데이터 불러오기 후처리
    func loadHandler(records: [CKRecord]?, error: Error?) {
        if error != nil {
            NSLog("자료 불러오기 실패. \(error)")
            tableView.reloadData()
            self.refreshControl?.endRefreshing()
        } else {
            if totalTodo == nil {
                // 앱 실행 시 처음으로 데이터를 로드하는 경우 전체 개수 저장
                totalTodo = records!.count
                self.refreshControl?.beginRefreshing()
            } else {
                // 서브 레코드인 경우이므로, 카운트 차감 - 마지막 서브 레코드 확인 목적
                totalTodo? -= 1
            }
            
            for record in records as [CKRecord]! {
                if record.recordType == "todoRecord" {
                    record.setValue(true, forKey: "isSaved")
                    todo.data.insert([record, [] as NSMutableArray] as NSArray, at: 0)
                    todoDataLoad(recordType: "todoSubRecord", record: record)
                } else if record.recordType == "todoSubRecord" {
                    if let todoRecord = todo.data.object(at: totalTodo!) as? NSArray {
                        record.setValue(true, forKey: "isSaved")
                        (todoRecord[1] as! NSMutableArray).add(record)
                    }
                }
            }
            
            if totalTodo == 0 {
                // 마지막 서브 레코드까지 읽어온 경우 테이블 뷰 새로고침
                tableView.reloadData()
                self.refreshControl?.endRefreshing()
            }
        }
    }
    
    // 존 사용 가능 여부 확인
    func checkZoneAvailable() {
        database.fetchAllRecordZones(completionHandler: zoneFetchHandler)
    }
    
    // 존 사용 가능 여부에 따라 존 생성
    func zoneFetchHandler(zones: [CKRecordZone]?, error: Error?) {
        if error != nil {
            NSLog("zone 정보를 읽어올 수 없습니다. \(error)")
        } else {
            var todoZoneIs = false
            var todoSubZoneIs = false
            
            for zone in zones as [CKRecordZone]! {
                if zone.zoneID.zoneName == "todoZone" {
                    todoZoneIs = true
                } else if zone.zoneID.zoneName == "todoSubZone" {
                    todoSubZoneIs = true
                }
            }
            
            if !todoZoneIs {
                zoneInit(zoneName: "todoZone")
            } else {
                NSLog("todoZone 사용 가능")
            }
            
            if !todoSubZoneIs {
                zoneInit(zoneName: "todoSubZone")
            } else {
                NSLog("todoSubZone 사용 가능")
            }
            if todoZoneIs && todoSubZoneIs {
                todoDataLoad(recordType: "todoRecord", record: nil)
            }
        }
    }
    
    // 존 생성
    func zoneInit(zoneName: String) {
        switch zoneName {
        case "todoZone":
            let todoZoneID = CKRecordZoneID(zoneName: "todoZone", ownerName: CKCurrentUserDefaultName)
            let todoZone = CKRecordZone(zoneID: todoZoneID)
            database.save(todoZone, completionHandler: createZoneComplete)
        case "todoSubZone":
            let todoSubZoneID = CKRecordZoneID(zoneName: "todoSubZone", ownerName: CKCurrentUserDefaultName)
            let todoSubZone = CKRecordZone(zoneID: todoSubZoneID)
            database.save(todoSubZone, completionHandler: createZoneComplete)
        default:
            NSLog("zone 이름을 확인하세요.")
        }
    }
    
    // 존 생성 시 후처리
    func createZoneComplete(zone: CKRecordZone?, error: Error?) -> Void{
        if error != nil {
            NSLog("\(zone!.zoneID.zoneName) 생성을 실패했습니다.")
        } else {
            NSLog("\(zone!.zoneID.zoneName) 생성을 성공했습니다.")
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        tableView.reloadData()
    }
    
    func iCloudAvailable() {
        let container = CKContainer.default()
        container.accountStatus {
            [weak self] (accountStatus: CKAccountStatus, error: Error?) in
            if accountStatus != CKAccountStatus.available {
                let alert = UIAlertController(title: "iCloud", message: "iCloud를 이용할 수 없습니다. 설정을 확인한 후 앱을 재실행해주세요.", preferredStyle: .alert)
                let action = UIAlertAction(title: "확인", style: .default, handler: nil)
                alert.addAction(action)
                self?.present(alert, animated: true, completion: nil)
            }
        }
    }
    
    // 새로운 셀 추가하기
    func addTodo() {
        let todoRecord = makeTodoRecord()
        
        if isSub {
            todoRecord.setValue(((todo.data[rowNum!] as! NSArray)[0] as! CKRecord).recordID.recordName, forKey: "ref")
            ((todo.data[rowNum!] as! NSArray)[1] as! NSMutableArray).insert(todoRecord, at: 0)
        } else {
            todo.data.insert([todoRecord,[] as NSMutableArray] as NSMutableArray, at: 0)
        }
        
        tableView.reloadData()
        self.refreshControl?.endRefreshing()
        showSaveBtn()
    }
    
    // 새로운 레코드를 만들어서 반환
    func makeTodoRecord() -> CKRecord {
        let uuid = NSUUID().uuidString
        
        var zoneID: CKRecordZoneID!
        var recordID: CKRecordID!
        var todoRecord: CKRecord!
        
        if isSub {
            zoneID = CKRecordZoneID(zoneName: "todoSubZone", ownerName: CKCurrentUserDefaultName)
            recordID = CKRecordID(recordName: uuid, zoneID: zoneID)
            todoRecord = CKRecord(recordType: "todoSubRecord", recordID: recordID)
            todoRecord.setValue("", forKey: "ref")
        } else {
            zoneID = CKRecordZoneID(zoneName: "todoZone", ownerName: CKCurrentUserDefaultName)
            recordID = CKRecordID(recordName: uuid, zoneID: zoneID)
            todoRecord = CKRecord(recordType: "todoRecord", recordID: recordID)
        }
        
        todoRecord.setValue("", forKey: "title")
        todoRecord.setValue(false, forKey: "isComplete")
        todoRecord.setValue(false, forKey: "isSaved")
        
        return todoRecord
    }
    
    // Todo 자료 저장 버튼 활성화
    func showSaveBtn() {
        if navigationItem.rightBarButtonItems == nil || navigationItem.rightBarButtonItems?.count == 0 {
            navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: UIBarButtonSystemItem.save, target: self, action: #selector(saveTodo))
        }
    }
    
    // Todo 자료 저장 및 버튼 비활성화
    func saveTodo() {
        for todoRecord in todo.data {
            for todoSubRecord in (todoRecord as! NSArray)[1] as! [CKRecord] {
                if !(todoSubRecord.value(forKey: "isSaved") as! Bool) {
                    database.save(todoSubRecord, completionHandler: saveHandler)
                }
            }
            
            database.save((todoRecord as! NSArray)[0] as! CKRecord, completionHandler: saveHandler)
        }
        
        navigationItem.rightBarButtonItems?.removeLast()
    }
    
    // 자료 저장 이후 처리
    func saveHandler(todoRecord: CKRecord?, error: Error?) {
        if error != nil {
            NSLog("저장을 실패했습니다. \(error)")
        } else {
            todoRecord!.setValue(true, forKey: "isSaved")
            NSLog("저장을 성공했습니다.")
        }
    }
    
    // 해당 테이블의 섹션 개수 - 필수 함수
    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    // 해당 섹션의 셀 개수 - 필수 함수
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if isSub {
            return ((todo.data[rowNum] as! NSArray)[1] as! NSMutableArray).count
        } else {
            return todo.data.count
        }
    }
    
    // 셀 생성과 반환 - 필수 함수
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "todoCell", for: indexPath) as! TodoCell
        let row: Int = indexPath.row
        
        var record: CKRecord!
        
        if isSub {
            record = ((todo.data[rowNum] as! NSArray)[1] as! NSMutableArray)[row] as! CKRecord
            cell.todoCount?.removeFromSuperview()
            cell.isSub = true
            cell.rowNum = rowNum
        } else {
            record = (todo.data[row] as! NSArray)[0] as! CKRecord
            cell.todoCount.setTitle("\(((todo.data[row] as! NSArray)[1] as! NSMutableArray).count)", for: UIControlState.normal)
        }
        
        cell.isComplete = record.value(forKey: "isComplete") as! Bool
        cell.tableViewController = self
        cell.todoTitle.text = record.value(forKey: "title") as? String
        
        if (cell.isComplete == true) {
            cell.todoTitle.textColor = UIColor.gray
        } else {
            cell.todoTitle.textColor = UIColor.black
        }
        
        cell.todoTitle.isEnabled = false
        
        // 더블 탭 제스처 인식
        let doubleTapGR = UITapGestureRecognizer(target: self, action: #selector(doubleTap))
        doubleTapGR.numberOfTapsRequired = 2
        cell.addGestureRecognizer(doubleTapGR)
        
        // 스와이프 제스처 인식
        let swipeRightGR = UISwipeGestureRecognizer(target: self, action: #selector(swipe))
        let swipeLeftGR = UISwipeGestureRecognizer(target: self, action: #selector(swipe))
        swipeRightGR.direction = UISwipeGestureRecognizerDirection.right
        cell.addGestureRecognizer(swipeRightGR)
        swipeLeftGR.direction = UISwipeGestureRecognizerDirection.left
        cell.addGestureRecognizer(swipeLeftGR)
        
        return cell
    }
    
    // 셀을 더블 탭 했을 때
    func doubleTap(sender: UITapGestureRecognizer) {
        if let indexPath = tableView.indexPathForSelectedRow {
            if let cell = tableView.cellForRow(at: indexPath) as? TodoCell {
                cell.todoTitle.isEnabled = true
                cell.todoTitle.becomeFirstResponder()
            }
        }
    }
    
    // 셀을 스와이프 할 경우
    func swipe(sender: UISwipeGestureRecognizer) {
        let point = sender.location(in: tableView)
        let indexPath = tableView.indexPathForRow(at: point)
        if sender.direction == .left {
            // Todo 삭제
            if isSub {
                database.delete(withRecordID: (((todo.data[rowNum!] as! NSArray)[1] as! NSMutableArray)[indexPath!.row] as! CKRecord).recordID, completionHandler: removeHandler)
                ((todo.data[rowNum!] as! NSArray)[1] as! NSMutableArray).removeObject(at: indexPath!.row)
            } else {
                database.delete(withRecordID: ((todo.data[indexPath!.row] as! NSArray)[0] as! CKRecord).recordID, completionHandler: removeHandler)
                for subTodoRecord in (todo.data[indexPath!.row] as! NSArray)[1] as! [CKRecord] {
                    database.delete(withRecordID: subTodoRecord.recordID, completionHandler: removeHandler)
                }
                todo.data.removeObject(at: indexPath!.row)
            }
        } else if sender.direction == .right {
            // Todo 완료
            var record: CKRecord!
            
            if isSub {
                record = ((todo.data[rowNum] as! NSArray)[1] as! NSMutableArray)[indexPath!.row] as! CKRecord
            } else {
                record = (todo.data[indexPath!.row] as! NSArray)[0] as! CKRecord
            }
            record.setValue(!(record.value(forKey: "isComplete") as! Bool), forKey: "isComplete")
        }
        tableView.reloadData()
    }
    
    // 데이터 삭제 시 후처리
    func removeHandler(recordID: CKRecordID?, error: Error?) -> Void {
        if error != nil {
            NSLog("\(error)")
        } else {
            NSLog("삭제를 성공했습니다.")
        }
    }
    
    // 다른 셀을 선택해서 기존 셀 선택이 해제될 경우
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if let cell = tableView.cellForRow(at: indexPath) as? TodoCell {
            cell.todoTitle.resignFirstResponder()
            cell.todoTitle.isEnabled = false
        }
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if let tableViewController = segue.destination as? TableViewController {
            if let indexPath = tableView.indexPathForSelectedRow {
                tableViewController.isSub = true
                tableViewController.rowNum = indexPath.row
            }
        }
    }
}
